from printmsg import PrintMsg
from lamb import Lambda
from config import Config
from os.path import basename
import os
import json
import yaml

"""
Default config yaml
"""
DEFAULT_CONFIG = {
    "Description": None,
    "FunctionName": None,
    "Handler": "lambda_handler",
    "MemorySize": 128,
    "Role": None,
    "Runtime": "python2.7",
    "Timeout": 15,
    "VpcConfig": None,
    "Environment": None,
    "KMSKeyArn": None,
    "TracingConfig": None,
    "DeadLetterConfig": None
}

"""
Default python source code
"""
DEFAULT_SOURCE = """from __future__ import print_function


# noinspection PyUnusedLocal
def lambda_handler(event, context):
    print(event)
"""

"""
Default Invoke config
"""
DEFAULT_INVOKE_CONFIG = {
    "FunctionName": None,
    "InvocationType": "Event",
    "LogType": "None",
    "ClientContext": None,
    "Payload": None,
    "Qualifier": None
}


class Project(object):
    """
    Project of lambda functions
    """
    path = None
    functions = []
    func = None
    libraries = None
    qualifier = None
    config_file = "config.yaml"
    i_config_file = "invoke.yaml"
    payload = None
    json_payload_file = None
    invoke_type = None
    virtual_env = None
    debug = False
    region = None
    dry = False
    config_postfix = '.yml'
    function_postfix = '.py'
    invoke_postfix = '-invoke.yml'

    def __init__(self, **kwargs):
        """
        Initialize project
        :param kwargs:
        :return:
        """
        if 'path' not in kwargs:
            raise KeyError('path is a Required Argument')
        else:
            self.path = kwargs['path']
        if 'qualifier' in kwargs:
            self.qualifier = kwargs['qualifier']
        if 'virtual_env' in kwargs:
            self.virtual_env = kwargs['virtual_env']
        if 'libraries' in kwargs:
            self.libraries = kwargs['libraries']
        if 'config_file' in kwargs:
            self.config_file = kwargs['config_file']
        if 'invoke_file' in kwargs:
            self.i_config_file = kwargs['invoke_file']
        if 'payload' in kwargs and kwargs['payload']:
            self.payload = self.load_json(kwargs['payload'])
        if 'invoke_type' in kwargs and kwargs['invoke_type']:
            self.invoke_type = kwargs['invoke_type']
        if 'debug' in kwargs:
            self.debug = kwargs['debug']
        if 'region' in kwargs:
            self.region = kwargs['region']
        if 'dry' in kwargs:
            self.dry = kwargs['dry']
        if 'func' in kwargs and kwargs['func']:
            self.func = kwargs['func']
        PrintMsg.debug = self.debug
        PrintMsg.cmd('Path {}'.format(self.path), 'INITIALIZING', 'yellow')
        if not self.func:
            self.initialize_functions()

    def initialize_functions(self):
        """
        Initialize list of functions
        """
        for root, dirs, files in os.walk(self.path):
            for f in files:
                if f.endswith(self.function_postfix):
                    file_name = os.path.splitext(basename(f))[0]
                    config = self.get_config(
                        root,
                        self.config_file,
                        file_name,
                        DEFAULT_CONFIG
                    )
                    icf = self.get_config(
                        root,
                        self.i_config_file,
                        file_name,
                        DEFAULT_INVOKE_CONFIG,
                        self.invoke_postfix
                    )
                    self.functions.append(
                        Lambda(
                            function=f,
                            function_name=file_name,
                            path=os.path.join(root),
                            virtual_env=self.virtual_env,
                            config=config,
                            qualifier=self.qualifier,
                            libraries=self.libraries,
                            region=self.region,
                            debug=self.debug,
                            dry=self.dry,
                            invoke_config=icf,
                            payload=self.json_payload_file,
                            invoke_type=self.invoke_type
                        )
                    )

    def get_config(self, path, config_file, name=None,
                   default=None, postfix='.yml'):
        """
        Load config yaml
        :param name:
        :param postfix:
        :param default:
        :param path:
        :param config_file:
        :return:
        """
        if os.path.exists(os.path.join(path, config_file)):
            cf = Config(os.path.join(path, config_file))
            data = cf.yaml_data
            if default:
                data = self.merge_config(data, default)
            return data
        elif name and os.path.exists(os.path.join(path, name) + postfix):
            cf = Config(os.path.join(path, name) + postfix)
            data = cf.yaml_data
            if default:
                data = self.merge_config(data, default)
            return data
        else:
            return default

    @staticmethod
    def merge_config(data, default):
        """
        Merge config data with default
        :param data:
        :param default:
        :return data:
        """
        for k, v in default.items():
            if k not in data:
                data[k] = v
        return data

    def load_json(self, payload):
        """
        Load json from payload file
        :param payload:
        :return rj:
        """
        rj = None
        if os.path.exists(os.path.join(self.path, payload)):
            self.json_payload_file = os.path.join(self.path, payload)
            with open(os.path.join(self.path, payload), 'r') as j:
                try:
                    rj = json.load(j)
                except TypeError:
                    PrintMsg.error('Invalid json payload')
        elif os.path.exists(payload):
            self.json_payload_file = payload
            with open(payload, 'r') as j:
                try:
                    rj = json.load(j)
                except TypeError:
                    PrintMsg.error('Invalid json payload')
        if self.debug:
            PrintMsg.out(rj)
        return rj

    def invoke(self, func):
        """
        Invoke a lambda function
        :param func:
        :return:
        """
        file_name = os.path.join(self.path, func)
        config = self.get_config(
            self.path,
            self.config_file,
            file_name,
            DEFAULT_CONFIG
        )
        icf = self.get_config(
            self.path,
            self.i_config_file,
            file_name,
            DEFAULT_INVOKE_CONFIG,
            self.invoke_postfix
        )
        Lambda(
            function=file_name,
            path=self.path,
            funcion_name=func,
            virtual_env=self.virtual_env,
            config=config,
            qualifier=self.qualifier,
            libraries=self.libraries,
            region=self.region,
            debug=self.debug,
            dry=self.dry,
            invoke_config=icf,
            payload=self.json_payload_file,
            invoke_type=self.invoke_type
        ).invoke()

    def invoke_all(self):
        """
        Invoke all functions in path
        :return:
        """
        for f in self.functions:
            f.invoke()
        PrintMsg.done('Invoking all')

    def deploy(self, func):
        """
        Deploy function
        :param func:
        :return:
        """
        file_name = os.path.join(self.path, func)
        config = self.get_config(
            self.path,
            self.config_file,
            file_name,
            DEFAULT_CONFIG
        )
        icf = self.get_config(
            self.path,
            self.i_config_file,
            file_name,
            DEFAULT_INVOKE_CONFIG,
            self.invoke_postfix
        )
        f = func + self.function_postfix
        Lambda(
            function=f,
            function_name=func,
            path=self.path,
            virtual_env=self.virtual_env,
            config=config,
            qualifier=self.qualifier,
            libraries=self.libraries,
            region=self.region,
            debug=self.debug,
            dry=self.dry,
            invoke_config=icf,
            payload=self.json_payload_file,
            invoke_type=self.invoke_type
        ).create()
        PrintMsg.done('Deploying')

    def deploy_all(self):
        """
        Deploy all functions in path
        :return:
        """
        for f in self.functions:
            f.create()
        PrintMsg.done('Deploying all')

    @staticmethod
    def new(**kwargs):
        """
        New function
        :param kwargs:
        :return:
        """
        if 'Path' not in kwargs or not kwargs['Path']:
            raise KeyError('path is a Required Argument')
        if 'Function' not in kwargs or not kwargs['Function']:
            raise KeyError('function is a Required Argument')
        PrintMsg.cmd('New lambda function {}.'.format(
            kwargs['Function']), 'INITIALIZING', 'yellow')
        path = kwargs['Path']
        func = kwargs['Function']
        kwargs.pop('Path', None)
        kwargs.pop('Function', None)
        cf = {k: v for k, v in kwargs.items() if v}
        cf = Project.merge_config(cf, DEFAULT_CONFIG)
        cf = {k: v for k, v in cf.items() if v}
        cf_name = os.path.join(path, func) + Project.config_postfix
        f_name = os.path.join(path, func) + Project.function_postfix
        PrintMsg.creating('Config file {}.'.format(cf_name))
        if not os.path.exists(cf_name):
            with open(cf_name, 'w') as j:
                yaml.safe_dump(cf, j, default_flow_style=False)
            PrintMsg.done('Creating config file {}.'.format(cf_name))
        else:
            PrintMsg.error('Config file already exists.')
        PrintMsg.creating('Source file {}'.format(f_name))
        if not os.path.exists(f_name):
            f = open(f_name, 'w')
            f.write(DEFAULT_SOURCE)
            f.close()
            PrintMsg.done('Creating source file {}.'.format(f_name))
        else:
            PrintMsg.error('File already exists.')
        PrintMsg.done('Creating lambda function {}.'.format(func))



import click


class PrintMsg(object):
    """
    Print messages
    """
    debug = True

    @staticmethod
    def creating(msg, cmd_color='green', color='white', bold=False):
        """
        Print creating
        :param cmd_color:
        :param msg:
        :param color:
        :param bold:
        :return:
        """
        PrintMsg.cmd(msg, 'CREATING', cmd_color, color, bold)

    @staticmethod
    def updating(msg, cmd_color='green', color='white', bold=False):
        """
        Print updating
        :param cmd_color:
        :param msg:
        :param color:
        :param bold:
        :return:
        """
        PrintMsg.cmd(msg, 'UPDATING', cmd_color, color, bold)

    @staticmethod
    def invoking(msg, cmd_color='blue', color='white', bold=False):
        """
        Print invoking
        :param cmd_color:
        :param msg:
        :param color:
        :param bold:
        :return:
        """
        PrintMsg.cmd(msg, 'INVOKING', cmd_color, color, bold)

    @staticmethod
    def error(msg, cmd_color='red', color='white', bold=False):
        """
        Print error
        :param msg:
        :param cmd_color:
        :param color:
        :param bold:
        :return:
        """
        PrintMsg.cmd(msg, 'ERROR', cmd_color, color, bold)

    @staticmethod
    def done(msg, cmd_color='green', color='white', bold=False):
        """
        Print done command
        :param msg:
        :param cmd_color:
        :param color:
        :param bold:
        :return:
        """
        PrintMsg.cmd(msg, 'DONE', cmd_color, color, bold)

    @staticmethod
    def out(msg, color='white', bold=False):
        """
        Print out message
        :param msg:
        :param color:
        :param bold:
        :return:
        """
        if PrintMsg.debug:
            click.echo(PrintMsg.colored_text(msg, color, bold))

    @staticmethod
    def cmd(msg, cmd, cmd_color='green', color='white', bold=False):
        """
        Print command
        :param msg:
        :param cmd:
        :param cmd_color:
        :param color:
        :param bold:
        """
        msg = "[{}] {}".format(PrintMsg.colored_text(cmd, cmd_color), msg)
        click.echo(PrintMsg.colored_text(msg, color, bold))

    @staticmethod
    def colored_text(text, color, bold=False):
        """
        Color text
        :param text:
        :param color:
        :param bold:
        :return:
        """
        return click.style(str(text), fg=color, bold=bold)

    @staticmethod
    def attr(atr, val, attr_color='green', val_color='blue'):
        """
        Print an attribute
        :param val:
        :param attr_color:
        :param atr:
        :param val_color:
        :return:
        """
        if PrintMsg.debug:
            click.echo('{}: {}'.format(
                PrintMsg.colored_text(atr, attr_color),
                PrintMsg.colored_text(val, val_color)
            ))



from os.path import basename
from printmsg import PrintMsg
from botocore.exceptions import ClientError
import concurrent.futures
import json
import zipfile
import base64
import boto3
import os


class Lambda(object):
    """
    Lambda Functions
    """
    zip_name = "deploy.zip"
    virtual_env = None
    config = None
    invoke_config = None
    function = None
    function_name = None
    qualifier = None
    libraries = None
    client = None
    path = None
    region = None
    debug = False
    dry = False
    runtime = None
    invoke_type = None
    payload = None

    def __init__(self, **kwargs):
        """
        Initialize lambda
        :param kwargs:
        :return:
        """
        if 'function' not in kwargs:
            raise KeyError('function is a Required Argument')
        else:
            self.function = kwargs['function']
        if 'function_name' not in kwargs:
            self.function_name = os.path.splitext(basename(self.function))[0]
        else:
            self.function_name = kwargs['function_name']
        if 'config' not in kwargs:
            raise KeyError('config is a Required Argument')
        else:
            self.config = kwargs['config']
        if 'path' not in kwargs:
            raise KeyError('path is a Required Argument')
        else:
            self.path = kwargs['path']
        if 'virtual_env' in kwargs and kwargs['virtual_env']:
            self.virtual_env = kwargs['virtual_env']
            if '~' in self.virtual_env:
                self.virtual_env = os.path.expanduser(self.virtual_env)
        if 'qualifier' in kwargs:
            self.qualifier = kwargs['qualifier']
        if 'invoke_type' in kwargs and kwargs['invoke_type']:
            self.invoke_type = kwargs['invoke_type']
        if 'payload' in kwargs and kwargs['payload']:
            self.payload = kwargs['payload']
        if 'debug' in kwargs:
            self.debug = kwargs['debug']
        if 'dry' in kwargs:
            self.dry = kwargs['dry']
        if 'invoke_config' in kwargs and kwargs['invoke_config']:
            self.invoke_config = kwargs['invoke_config']
        if not self.payload and 'Payload' in self.invoke_config:
            self.payload = self.invoke_config['Payload']
        PrintMsg.debug = self.debug
        self.libraries = []
        if 'libraries' in kwargs and kwargs['libraries']:
            if ',' in kwargs['libraries']:
                libs = kwargs['libraries'].split(',')
                for lib in libs:
                    if '~' in lib:
                        self.libraries.append(os.path.expanduser(lib))
                    else:
                        self.libraries.append(lib)
            else:
                lib = kwargs['libraries']
                if '~' in lib:
                    lib = os.path.expanduser(lib)
                self.libraries.append(lib)
        self.zip_name = 'deploy-{}.zip'.format(self.function)
        if 'FunctionName' in self.config and self.config['FunctionName']:
            self.function_name = self.config['FunctionName']
        if 'region' in kwargs and kwargs['region']:
            self.region = kwargs['region']
        elif 'region' in self.config and self.config['region']:
            self.region = self.config['region']
        if 'runtime' in kwargs:
            self.runtime = kwargs['runtime']
        elif 'Runtime' in self.config:
            self.runtime = self.config['Runtime']
        if self.qualifier:
            self.invoke_config['Qualifier'] = self.qualifier
        if self.payload:
            self.invoke_config['Payload'] = self.__read_payload__()
        if self.dry:
            self.invoke_config['InvocationType'] = 'DryRun'
        elif self.invoke_type:
            self.invoke_config['InvocationType'] = self.invoke_type
        if 'FunctionName' not in self.invoke_config:
            self.invoke_config['FunctionName'] = self.function_name
        if not self.invoke_config['FunctionName']:
            self.invoke_config['FunctionName'] = self.function_name
        if '.' not in self.config['Handler']:
            fn = self.function.split('.')[0]
            self.config['Handler'] = '{}.{}'.format(fn, self.config['Handler'])
        self.invoke_config = {k: v for k, v in self.invoke_config.items() if v}
        self.config = {k: v for k, v in self.config.items() if v}
        self.client = boto3.client('lambda',  region_name=self.region)

    def create(self):
        """
        Create a new lambda function or update
        an existing function.
        :return:
        """
        try:
            PrintMsg.out(self.get())
            PrintMsg.updating('{} in region {}'.format(
                self.function_name, self.region))
            self.__update__()
        except ClientError as e:
            PrintMsg.out(e)
            PrintMsg.creating(self.function_name)
            self.__create__()

    def update(self):
        """
        Update a lambda function
        :return:
        """
        self.create()

    def get(self):
        """
        Get lambda function
        :return:
        """
        if self.qualifier:
            response = self.client.get_function(
                FunctionName=self.function_name,
                Qualifier=self.qualifier
            )
        else:
            response = self.client.get_function(
                FunctionName=self.function_name
            )
        return response

    def invoke(self):
        """
        Invoke lambda function
        :return:
        """
        try:
            PrintMsg.out(self.get())
            PrintMsg.invoking('{} in region {}'.format(
                self.function_name, self.region))
            self.__invoke__()
        except ClientError as e:
            PrintMsg.out(e)
            PrintMsg.error('Unable to invoke a function that does not exist')

    def __details__(self):
        """
        Show details of lambda function
        :return:
        """
        PrintMsg.out('{')
        PrintMsg.attr('  path', self.path)
        PrintMsg.attr('  function_name', self.function_name)
        PrintMsg.attr('  function', self.function)
        PrintMsg.attr('  qualifier', self.qualifier)
        PrintMsg.attr('  virtual_env', self.virtual_env)
        PrintMsg.attr('  libraries', self.libraries)
        PrintMsg.attr('  region', self.region)
        PrintMsg.attr('  config', self.config)
        PrintMsg.attr('  invoke_type', self.invoke_type)
        PrintMsg.attr('  invoke_config', self.invoke_config)
        PrintMsg.out('}')

    def __invoke__(self):
        """
        Invoke by configuration
        :return:
        """
        self.__details__()
        if self.invoke_type:
            invoke_type = self.invoke_type
        else:
            invoke_type = self.invoke_config['InvocationType']
        response = None
        if invoke_type == "RequestResponse":
            with concurrent.futures.ThreadPoolExecutor(
                    max_workers=5) as executor:
                futures = {executor.submit(
                    self.client.invoke, **self.invoke_config)}
                for future in concurrent.futures.as_completed(futures):
                    try:
                        response = future.result()
                        PrintMsg.cmd('Result', 'PAYLOAD')
                        payload = response['Payload'].read()
                        parsed = json.loads(payload)
                        print(json.dumps(parsed, indent=4, sort_keys=True))
                    except Exception as exc:
                        PrintMsg.error(exc)
        else:
            response = self.client.invoke(**self.invoke_config)
        if response:
            if self.debug:
                PrintMsg.out(response)
            if 'StatusCode' in response:
                PrintMsg.cmd(response['StatusCode'], 'STATUS')
            if 'LogResult' in response:
                PrintMsg.cmd("", "LOG")
                print('')
                print(base64.b64decode(response['LogResult']))

    def __update__(self):
        """
        Update function code and properties
        :return:
        """
        self.__details__()
        self.__zip_function__()
        response = self.client.update_function_code(
            FunctionName=self.function_name,
            ZipFile=self.__read_zip__(),
            DryRun=self.dry
        )
        if self.debug:
            PrintMsg.out(response)
        PrintMsg.cmd('Sha256: {}'.format(
            response['CodeSha256']), 'UPDATED CODE')
        self.__delete_zip__()
        if not self.dry:
            response = self.client.update_function_configuration(
                **self.config)
            if self.debug:
                PrintMsg.out(response)
            PrintMsg.cmd('Sha256: {}'.format(
                response['CodeSha256']), 'UPDATED CONFIG')

    def __create__(self):
        """
        Create function code and properties
        :return:
        """
        self.__details__()
        self.__zip_function__()
        if not self.dry:
            self.config['Code'] = {"ZipFile": self.__read_zip__()}
            response = self.client.create_function(**self.config)
            if self.debug:
                PrintMsg.out(response)
            PrintMsg.cmd(response['CodeSha256'], 'CREATED')
            self.config.pop('Code', None)
        self.__delete_zip__()

    @staticmethod
    def zip_add_dir(path, zipf, keep_parent=False):
        """
        Add directory to zip file
        :param path:
        :param zipf:
        :param keep_parent:
        :return:
        """
        base = basename(os.path.dirname(path))
        if keep_parent:
            len_dir_path = len(path)-len(base)
        else:
            len_dir_path = len(path)
        for root, _, files in os.walk(path):
            for f in files:
                file_path = os.path.join(root, f)
                zipf.write(file_path, file_path[len_dir_path:])

    def __zip_function__(self):
        """
        Zip source code
        :return:
        """
        PrintMsg.cmd('{}'.format(
            os.path.join(self.path, self.zip_name)), 'ARCHIVING')
        zipf = zipfile.ZipFile(
            os.path.join(self.path, self.zip_name), 'w', zipfile.ZIP_DEFLATED)
        if self.virtual_env:
            env_path = self.virtual_env
            for root, dirs, files in os.walk(self.virtual_env):
                for d in dirs:
                    if d == 'site-packages':
                        env_path = os.path.join(root, d)
            Lambda.zip_add_dir(env_path, zipf)
        if len(self.libraries) > 0:
            for lib in self.libraries:
                Lambda.zip_add_dir(lib, zipf, True)
        zipf.write(os.path.join(self.path, self.function), self.function)
        zipf.close()

    def __read_zip__(self):
        """
        Read zip file returning bytes
        :return bytes:
        """
        return self.__read_file__(self.zip_name)

    def __read_payload__(self):
        """
        Read payload json file
        :return bytes:
        """
        return self.__read_file__(self.payload)

    def __read_file__(self, file_name):
        """
        Read a file and return bytes
        :param file_name:
        :return bts:
        """
        with open(os.path.join(self.path, file_name), 'rb') as zf:
            bts = zf.read()

        return bts

    def __delete_zip__(self):
        """
        Delete zip file of source
        :return:
        """
        if os.path.exists(os.path.join(self.path, self.zip_name)):
            PrintMsg.out('Deleting {}'.format(
                os.path.join(self.path, self.zip_name)))
            os.remove(os.path.join(self.path, self.zip_name))



from project import Project
import click
import os


@click.group(invoke_without_command=True)
@click.pass_context
def cli(ctx):
    """
    AWS Lambda Build Tool

    Build and deploy projects that use AWS Lambda.
    """
    if ctx.invoked_subcommand is None:
        print(ctx.get_help())


@cli.command()
def version():
    """
    Display albt version
    """
    with open(os.path.abspath(os.path.join(os.path.dirname(
            os.path.dirname(__file__)), 'VERSION'))) as version_file:
        v = version_file.read().strip()
    click.echo(click.style('albt: ', fg='white') + click.style(v, fg='green'))


@cli.command()
@click.argument('path', nargs=1)
@click.argument('func', nargs=-1)
@click.option('--qualifier', default=None,
              help='Qualifier example: dev|prod|latest')
@click.option('--region', default=None, help='AWS Region override')
@click.option('--venv', default=None, help='Included Virtual environment')
@click.option('--libs', default=None, help='Include library folders')
@click.option('--debug/--no-debug', default=False,
              help='Show Debugging information')
@click.option('--dry/--no-dry', default=False, help='Dry Run')
def deploy(path, func, qualifier, region, venv, libs, debug, dry):
    """
    Deploy lambda functions
    """
    p = Project(
        path=path,
        func=func,
        qualifier=qualifier,
        virtual_env=venv,
        libraries=libs,
        debug=debug,
        region=region,
        dry=dry
    )

    if func:
        p.deploy(''.join(func))
    else:
        p.deploy_all()


@cli.command()
@click.argument('path', nargs=1)
@click.argument('func', nargs=-1)
@click.option('--payload', default=None, help='Invoke payload')
@click.option('--invocation', default=None, help='Invocation type')
@click.option('--qualifier', default=None,
              help='Qualifier example: dev|prod|latest')
@click.option('--region', default=None, help='AWS Region override')
@click.option('--venv', default=None, help='Included Virtual environment')
@click.option('--libs', default=None, help='Include library folders')
@click.option('--debug/--no-debug', default=False,
              help='Show Debugging information')
@click.option('--dry/--no-dry', default=False, help='Dry run')
@click.option('--d/--no-d', default=False, help='Also deploy')
def invoke(path, func, payload, invocation, qualifier,
           region, venv, libs, debug, dry, d):
    """
    Invoke lambda functions
    """
    p = Project(
        path=path,
        func=func,
        virtual_env=venv,
        libraries=libs,
        qualifier=qualifier,
        debug=debug,
        region=region,
        dry=dry,
        payload=payload,
        invoke_type=invocation
    )

    if func:
        if d:
            p.deploy(''.join(func))
        p.invoke(''.join(func))
    else:
        if d:
            p.deploy_all()
        p.invoke_all()


@cli.command()
@click.argument('path', nargs=1)
@click.argument('func', nargs=1)
@click.option('--handler', default=None, help='Function handler name')
@click.option('--role', default=None, help='AWS Role')
@click.option('--memory', default=None, help='Memory Size')
@click.option('--timeout', default=None, help='Timeout')
@click.option('--description', default=None, help='Description')
@click.option('--runtime', default=None, help='Runtime')
@click.option('--name', default=None, help='Override function name')
@click.option('--region', default=None, help='AWS Region override')
@click.option('--venv', default=None, help='Included Virtual environment')
@click.option('--libs', default=None, help='Include library folders')
@click.option('--qualifier', default=None,
              help='Qualifier example: dev|prod|latest')
@click.option('--debug/--no-debug', default=False,
              help='Show Debugging information')
@click.option('--dry/--no-dry', default=False, help='Dry run')
@click.option('--d/--no-d', default=False, help='Also deploy')
def new(path, func, handler, role, memory, timeout,
        description, runtime, name, region, venv, libs,
        qualifier, debug, dry, d):
    """
    Create new lambda function
    """
    Project.new(
        Path=path,
        Function=func,
        FunctionName=name,
        Handler=handler,
        Role=role,
        MemorySize=memory,
        Timeout=timeout,
        Description=description,
        Tuntime=runtime,
        Region=region
    )

    if d:
        Project(
            path=path,
            qualifier=qualifier,
            virtual_env=venv,
            libraries=libs,
            debug=debug,
            region=region,
            dry=dry
        ).deploy(func)



import yaml


class Config(object):
    """
    Load Config yaml file
    """
    def __init__(self, yaml_file):
        """
        Initialize config yaml
        :param yaml_file:
        :return:
        """
        with open(yaml_file, 'r') as stream:
            try:
                self.yaml_data = yaml.load(stream)
            except yaml.YAMLError as exc:
                print(exc)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Setuptools module for albt
"""
from setuptools import setup, find_packages
from codecs import open
import os

with open(os.path.abspath(os.path.join(os.path.dirname(__file__), 'README.rst'))) as f:
    long_description = f.read()

with open(os.path.abspath(os.path.join(os.path.dirname(__file__), 'VERSION'))) as version_file:
    version = version_file.read().strip()

setup(
    name='albt',
    version=version,
    url='https://github.com/geothird/albt',
    description='AWS Lambda Build tool',
    long_description=long_description,
    author='geothird',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'
    ],
    keywords='aws lambda python',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=[
        'Click',
        'boto3',
        'pyyaml'
    ],
    entry_points='''
        [console_scripts]
        albt=albt.albt:cli
    ''',
)
